#ifndef NDEBUG
#define NDEBUG
#endif

using namespace std;

#include <cstdio>
#include <cstring>
#include <vector>
#include <cassert>

#define FIN  "import.in"
#define FOUT "import.out"

#define pb push_back

#define MAX_N 256
#define MAX_V 100000
#define INFTY 0x3f3f3f3f

int N, M, K;
int U[MAX_N], fat[MAX_N];

vector<int> G[MAX_N];
vector<int> adj[MAX_N];

int D[MAX_N][MAX_N];

void add_edge(int a, int b)
{
	if (b == 1)
		a ^= b ^= a ^= b;
	if (a == 1) {
		if (b > K) {
			G[0].pb(b);
			G[b].pb(0);
		} else {
			G[1].pb(b);
			G[b].pb(1);
		}
	} else {
		G[a].pb(b);
		G[b].pb(a);
	}
}

void read()
{
	int i, a, b, c, x;
	FILE *fi = fopen(FIN, "r");
	fscanf(fi, "%d%d%d", &N, &M, &K);
	for (i = 1; i < N; ++i) {
		fscanf(fi, "%d%d", &a, &b);
		add_edge(a, b);
	}
	while (M--) {
		fscanf(fi, "%d%d%d%d", &a, &b, &c, &x);
		if (a > b)
			a ^= b ^= a ^= b;
		if (x == 0) {
			adj[a].pb(b), D[a][b] = -c;
		} else {
			adj[b].pb(a), D[b][a] = c-1;
		}
	}
	G[0].pb(1), G[1].pb(0);
	fclose(fi);
}

void DFS0(int n)
{
	vector<int>::iterator it;
	for (U[n] = 1, it = G[n].begin(); it != G[n].end(); ++it)
		if (!U[*it]) {
			fat[*it] = n;
			DFS0(*it);
		}
}

#define QSZ 255

int dist[MAX_N];
int Q[QSZ+5], enq[MAX_N];

void BKL()
{
	int qb, qe, n, i;
	vector<int>::iterator it;
	memset(dist, INFTY, sizeof(dist));
	dist[0] = 0;

	for (enq[ Q[ qb = qe = 0 ] = 0 ] = 1; qb <= qe; enq[n] = 0, ++qb)
		for (n = Q[qb], it = adj[n].begin(); it != adj[n].end(); ++it)
			if (dist[n]+D[n][*it] < dist[*it]) {
				dist[*it] = dist[n]+D[n][*it];
				if (!enq[*it]) {
					Q[ (++qe) & QSZ ] = *it;
					enq[*it] = 1;
				}
			}
	for (i = 0; i <= N; ++i)
		for (it = adj[i].begin(); it != adj[i].end(); ++it)
			assert(dist[i]+D[i][*it] >= dist[*it]);
}

void solve()
{
	int i, j;
	vector<int>::iterator it;
	DFS0(0);
	for (i = 1; i <= N; ++i) {
		adj[i].pb(fat[i]), adj[fat[i]].pb(i);
		D[i][fat[i]] = D[fat[i]][i] = MAX_V;
	}
	BKL();
}

void write()
{
	int i;
	FILE *fo = fopen(FOUT, "w");
	for (i = 1; i <= K; ++i)
		fprintf(fo, "%d ", dist[i]-dist[fat[i]]);
	for (i = K+1; i < N; ++i)
		fprintf(fo, "%d ", dist[fat[i]]-dist[i]);
	fprintf(fo, "%d\n", dist[fat[N]]-dist[N]);
	fclose(fo);
}

int main()
{
	read();
	solve();
	write();
	return 0;
}
